package com.emato.ccnet.biz.req.order.csp;

import com.emato.ccnet.biz.req.CebCustomsResponse;
import com.emato.ccnet.contants.Contants;
import com.emato.ccnet.dto.*;
import com.emato.ccnet.domain.ccnet.CusOrder;
import com.emato.ccnet.domain.ccnet.CusOrderResp;
import com.emato.ccnet.domain.ccnet.CusOrderStatus;
import com.emato.ccnet.message.Result;
import com.emato.ccnet.service.ccent.CusOrderRespService;
import com.emato.ccnet.service.ccent.CusOrderService;
import com.emato.ccnet.service.ccent.CusOrderStatusService;
import com.emato.ccnet.service.kmall.KmallOrderService;
import com.emato.ccnet.util.JacksonUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.ImmutableMap;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public class Ceb311CustiomsResponse implements CebCustomsResponse{
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311CustiomsResponse.class);

    @Autowired
    private CusOrderRespService cusOrderRespService;
    @Autowired
    private CusOrderStatusService cusOrderStatusService;
    @Autowired
    private KmallOrderService kmallOrderService;
    @Autowired
    private CusOrderService cusOrderService;

    @Override
    public void biz(Map<String, Object> map) {
        //解析数据
        OrderCustiomsResponse orderCustiomsResponse = analysisCustiomsResponse(map);
        if(orderCustiomsResponse == null){
            LOGGER.info("解析响应数据失败");
            return;
        }
        OrderDto orderDto = orderCustiomsResponse.getClsOrderDto();
        if(orderDto == null){
            LOGGER.info("csp返回订单数据获取失败");
            return;
        }
        String orderSn = orderDto.getOrderSn();
        String orderNo = orderDto.getOrderNo();

        //查找电子订单
        CusOrder cusOrder = cusOrderService.findCusOrder(orderSn);
        if(cusOrder == null){
            LOGGER.info("查找电子订单表数据失败");
            return;
        }

        //将电子订单回执初始化
        CusOrderResp cusOrderResp = cusOrderRespService.insertCusOrderRespByCustioms(orderCustiomsResponse,cusOrder.getCusOrderSn());
        if(cusOrderResp == null){
            LOGGER.info("电子订单海关回执初始化数据失败");
            return;
        }

        //查找电子订单状态表
        Map<String, Object> reqParam = ImmutableMap.of("orderSn", cusOrderResp.getOrderSn(),"sendStatus",Contants.DECLARATION_SUCCESS_STATUS);
        CusOrderStatus cusOrderStatus = cusOrderStatusService.findCusOrderStatus(reqParam);
        if(cusOrderStatus == null){
            LOGGER.info("查找电子订单状态表数据失败");
            return;
        }

        //对响应的数据进行处理
        if(Contants.ORDER_RESPONSE_SUCCESS.equals(orderCustiomsResponse.getCode())){
            cusOrderStatus.setSendStatus(Contants.SUCCESS_STATUS);
            Map<String, Object> kmallMap = ImmutableMap.of("orderNo", orderNo, "cusOrderFlag", Contants.SUCCESS_STATUS);
            kmallOrderService.updateCusKmallOrderStatus(kmallMap);
        }

        if(Contants.ORDER_RESPONSE_FAIL.equals(orderCustiomsResponse.getCode())){
            cusOrderStatus.setSendStatus(Contants.FAIL_STATUS);
            Map<String, Object> kmallMap = ImmutableMap.of("orderNo", orderNo, "cusOrderFlag", Contants.FAIL_STATUS);
            kmallOrderService.updateCusKmallOrderStatus(kmallMap);
        }

        //对回调报文进行处理:平台回执
        if(orderCustiomsResponse.getEportResultStatusDto() != null){
            EportResultStatusDto eportResultStatusDto = orderCustiomsResponse.getEportResultStatusDto();
            if(Contants.ORDER_PLAT_SUCCESS.equals(eportResultStatusDto.getCode())){
                cusOrderStatus.setPlatRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.ORDER_PLAT_FAIL.equals(eportResultStatusDto.getCode())){
                cusOrderStatus.setPlatRespStatus(Contants.FAIL_STATUS);
            }
        }

        //海关回执
        if(orderCustiomsResponse.getCusResultStatusDto() != null){
            CusResultStatusDto cusResultStatusDto = orderCustiomsResponse.getCusResultStatusDto();
            if(Contants.CUS_DECLARATION.equals(cusResultStatusDto.getCode())){
                cusOrderStatus.setCusRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.CUS_CHARGEBACK.equals(cusResultStatusDto.getCode()) || Contants.CUS_FAIL.equals(cusResultStatusDto.getCode())){
                cusOrderStatus.setCusRespStatus(Contants.FAIL_STATUS);
                cusOrder.setAppType(Contants.APP_STATUS_SEPARETE);
            }
        }

        //国检回执
        if(orderCustiomsResponse.getCiqResultStatusDto() != null){
            CiqResultStatusDto ciqResultStatusDto = orderCustiomsResponse.getCiqResultStatusDto();
            if(Contants.CIQ_SUCCESS.equals(ciqResultStatusDto.getCode())){
                cusOrderStatus.setCipRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.CIQ_FAIL.equals(ciqResultStatusDto.getCode())||Contants.CIQ_RECEIVE_FAIL.equals(ciqResultStatusDto.getCode())){
                cusOrderStatus.setCipRespStatus(Contants.FAIL_STATUS);
                cusOrder.setAppType(Contants.APP_STATUS_SEPARETE);
            }
        }

        int cusOrderSta = cusOrderStatusService.updateCusOrderStatus(cusOrderStatus);
        if(cusOrderSta != 1){
            LOGGER.info("电子订单海关回执更新数据失败");
            return;
        }

        int cusOr = cusOrderService.updateCusOrder(cusOrder);
        if(cusOr != 1){
            LOGGER.info("电子订单海关回执更新数据失败");
            return;
        }

        LOGGER.info("海关处理回调成功");
    }

    /**
     * 解析海关回调数据
     * @param map
     * @return
     */
    private OrderCustiomsResponse analysisCustiomsResponse(Map<String, Object> map) {
        Gson gson = new Gson();
        Result response = (Result) map.get("resultData");

        if(response == null){
            LOGGER.info("解析响应数据失败");
            return null;
        }

        //解析回执数据
        String data = JacksonUtil.toJson(response.getData());

        LOGGER.info("海关回执 data:"+data);

        List<OrderCustiomsResponse> orderCustiomsResponseList  = gson.fromJson(data, List.class);

        if(orderCustiomsResponseList == null && orderCustiomsResponseList.size() <= 0){
            LOGGER.info("解析响应数据失败");
            return null;
        }

        return gson.fromJson(gson.toJson(orderCustiomsResponseList.get(0)), OrderCustiomsResponse.class);
    }
}
