package com.emato.ccnet.biz.req.order.csp;

import com.emato.ccnet.biz.req.CebReqBiz;
import com.emato.ccnet.biz.req.CebReqSendDate;
import com.emato.ccnet.msg.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Handler;

@Component
public class Ceb311OrderBiz implements CebReqBiz {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311OrderBiz.class);
    private Map<String, Object> param = Collections.EMPTY_MAP;

    @Autowired
    private Ceb311OrderGetDate ceb311OrderGetDate;

    @Autowired
    private Ceb311OrderAddDate ceb311OrderAddDate;

    @Autowired
    private Ceb311SendDateRequet ceb311SendDateRequet;

    @Value("${db.ceb311-send-csp.data-size}")
    private String limit = "20";
    /**
     * 电子订单正常业务入口
     * @return
     */
    @Override
    public ResponseMessage biz() {

        /*if (reqParam.isEmpty()) {
            String info = "【CEB411】请求订单支付数据,入参为空";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }*/

        Map<String,Object> reqParam = ImmutableMap.of("number", limit);
        //查询中网电子订单数据
        List list = ceb311OrderGetDate.getData(reqParam);
        if (list.isEmpty()) {
            String info = "【CEB311】没有订单数据, 生成海关支付单报文提前结束";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //添加erp电子订单数据
        ResponseMessage responseMessage = ceb311OrderAddDate.addDate(list);
        if(!ResponseMessage.isSuccess(responseMessage)){
            String info = "生成海关支付单初始化失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送数据csp系统请求并响应更新订单
        ResponseMessage ceb311ResponseMessage = ceb311SendDateRequet.sendDate();
        if(!ResponseMessage.isSuccess(ceb311ResponseMessage)){
            String info = "发送请求响应失败"+ceb311ResponseMessage.getMsg();
            LOGGER.error("--- {}", ceb311ResponseMessage.getMsg());
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();
    }

    public Map<String, Object> getParam() {
        return param;
    }

    public void setParam(Map<String, Object> param) {
        this.param = param;
    }

}
