package com.emato.ccnet.biz.req.pay.alipay;

import com.emato.ccnet.biz.req.CebReqBiz;
import com.emato.ccnet.msg.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * 海关支付单 业务
 * @author Scott Chen
 * @version 1.0
 * 2017-11-07 10:10
 */
@Component
public class Ceb411AlipayBiz implements CebReqBiz {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb411AlipayBiz.class);

    @Autowired
    private Ceb411AlipayGetDate ceb411AlipayGetData;

    @Autowired
    private Ceb411AlipayAddData ceb411AlipayAddData;

    @Autowired
    private Ceb411SendDateRequet ceb411SendDateRequet;

    private Map<String, Object> param = Collections.EMPTY_MAP;


    /**
     * 支付订单业务入口
     * @return
     */
    @Override
    public ResponseMessage biz() {
       /* if (param.isEmpty()) {
            String info = "【CEB411】请求订单支付数据,入参为空";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }*/
        Map<String,Object> reqParam = ImmutableMap.of("number", 0);
        //查询支付单数据
        List list = ceb411AlipayGetData.getData(reqParam);
        if (list.isEmpty()) {
            String info = "【CEB411】没有订单数据, 生成海关支付单报文提前结束";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //添加支付表数据
        ResponseMessage responseMessage = ceb411AlipayAddData.addDate(list);
        if(!ResponseMessage.isSuccess(responseMessage)){
            String info = "生成海关支付单初始化失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送数据请求并响应
        responseMessage = ceb411SendDateRequet.sendDate();
        if(!ResponseMessage.isSuccess(responseMessage)){
            String info = "发送请求响应失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();
    }


    public Map<String, Object> getParam() {
        return param;
    }

    public void setParam(Map<String, Object> param) {
        this.param = param;
    }

}
