package com.emato.ccnet.biz.req.pay.alipay;

import com.emato.ccnet.biz.req.CebReqSendDate;
import com.emato.ccnet.contants.Contants;
import com.emato.ccnet.config.Cus;
import com.emato.ccnet.domain.ccnet.CusAlipay;
import com.emato.ccnet.domain.ccnet.CusAlipayStatus;
import com.emato.ccnet.msg.ResponseMessage;
import com.emato.ccnet.service.ccent.CusAlipayService;
import com.emato.ccnet.service.ccent.CusAlipayStatusService;
import com.emato.ccnet.util.AlipayCore;
import com.emato.ccnet.util.OkHttpUtil;
import com.emato.ccnet.util.sign.RSA;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;

/**
 * 支付请求信息
 */
@Component
@Transactional(value = "secondTransactionManager")
public class Ceb411SendDateRequet implements CebReqSendDate {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb411SendDateRequet.class);

    @Autowired
    private CusAlipayService cusAlipayService;

    @Autowired
    private CusAlipayStatusService cusAlipayStatusService;

    @Autowired
    private Ceb411SendDateAfter ceb411SendDateAfter;

    @Autowired
    private Ceb411SendDateResponse ceb411SendDateResponse;

    @Autowired
    private Cus cus;
    /**
     * 支付数据交互
     * @return
     */
    public ResponseMessage sendDate() {

        //取支付单数据
        List<CusAlipay> cusAlipayList = cusAlipayService.getCusAlipay();
        if (cusAlipayList == null || cusAlipayList.size() == 0) {
            String info = "查找当前ccnet中的所有支付单数据失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送给第三方的请求
        int alipaySumbit = buildRequest(cusAlipayList);
        if(alipaySumbit != 1){
            String info = "支付订单发送给第三方的请求失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();

    }

    /**
     * 发送给第三方的请求
     * @param cusAlipayList
     * @return
     */
    private int buildRequest(List<CusAlipay> cusAlipayList) {

        List<String> sucList = new ArrayList<>();
        List<String> failedList = new ArrayList<>();

        for (CusAlipay cusAlipay : cusAlipayList) {

            //更新支付状态表变成发送中
            int update = updateAlipayStatus(cusAlipay.getCusPaySn());
            if(update != 1){
                failedList.add(cusAlipay.getOrderSn());
                String info = "更新支付状态表变成发送中失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //构造请求支付宝的参数数值
            Map<String, String> sPara = buildAlipayMap(cusAlipay);
            if(sPara == null){
                failedList.add(cusAlipay.getOrderSn());
                String info = "构造请求支付宝的参数数值失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //发送请求并获取响应信息
            String response = requestSubmit(sPara);
            LOGGER.info("支付宝的响应报文：" + response);
            if(response == null){
                failedList.add(cusAlipay.getOrderSn());
                String info = "发送请求并获取响应信息失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //解析响应信息并处理
            int addRespone = ceb411SendDateResponse.addResponse(response,cusAlipay);
            if(addRespone <= 0){
                failedList.add(cusAlipay.getOrderSn());
                String info = "解析响应信息并处理失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            sucList.add(cusAlipay.getOrderSn());
        }

        LOGGER.info("--- 【CEB411】【成功】订单数:{}, 订单: {}", sucList.size(), Joiner.on(",").skipNulls().join(sucList.toArray()));
        LOGGER.info("--- 【CEB411】【失败】订单数:{}, 订单: {}", failedList.size(), Joiner.on(",").skipNulls().join(failedList.toArray()));

        ceb411SendDateAfter.afterSend(sucList,failedList);


        return 0;
    }

    /**
     * 更新支付宝状态表状态变成发送中
     * @param cusPaySn
     * @return
     */
    private int updateAlipayStatus(String cusPaySn) {
        Map<String, Object> map = ImmutableMap.of("cusPaySn", cusPaySn, "sendStatus", Contants.INIT_STATUS);
        //通过编号查找支付单阿里状态表
        CusAlipayStatus cusAlipayStatus = cusAlipayStatusService.findCusAlipayStatus(map);
        if(cusAlipayStatus == null){
            return  0;
        }

        //更新支付单阿里状态
        cusAlipayStatus.setSendStatus(Contants.SENDING_STATUS);
        cusAlipayStatus.setModerSn(Contants.CREATER_NUMBER);
        cusAlipayStatus.setModTime(new Date());
        int status = cusAlipayStatusService.updateCusAlipayStatus(cusAlipayStatus);
        if(status <= 0){
            return 0;
        }

        return 1;
    }

    /**
     * 构造支付宝请求参数
     * @param cusAlipay
     * @return
     */
    private Map<String,String> buildAlipayMap(CusAlipay cusAlipay) {
        Map<String, String> sParaTemp = new TreeMap<String, String>();
        sParaTemp.put("service", "alipay.acquire.customs");
        sParaTemp.put("partner", cus.getAlipay().get("partner"));
        sParaTemp.put("_input_charset",  cus.getAlipay().get("input-charset"));
        sParaTemp.put("out_request_no", cusAlipay.getOutRequestNo());
        sParaTemp.put("trade_no", cusAlipay.getTradeNo());
        sParaTemp.put("merchant_customs_code", cusAlipay.getMerchantCustomsCode());
        sParaTemp.put("merchant_customs_name", cusAlipay.getMerchantCustomsName());
        sParaTemp.put("amount", cusAlipay.getAmount()+"");
        sParaTemp.put("customs_place", cusAlipay.getCustomsPlace());

        //生成要请求给支付宝的参数数组
        Map<String, String> sPara = buildRequestPara(sParaTemp);
        return sPara;
    }


    /**
     * 构建请求html并响应数据
     * @param sPara
     * @return
     */
    private String requestSubmit(Map<String, String> sPara) {
        //构造RequestBody
        RequestBody requestBody = OkHttpUtil.buildRequestBody(sPara);

        //构建Request
        String url = cus.getAlipay().get("getWay");
        Request request = OkHttpUtil.buildRequest(url,requestBody);

        LOGGER.info("支付宝的请求报文：" + request);
        // 同步访问,返回结果字符串
        String response = null;
        try {
            response = OkHttpUtil.post(request);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return response;
    }

    /**
     * 生成要请求给支付宝的参数数组
     * @param sParaTemp 请求前的参数数组
     * @return 要请求的参数数组
     */
    private TreeMap<String,String> buildRequestPara(Map<String, String> sParaTemp) {
        //除去数组中的空值和签名参数
        TreeMap<String, String> sPara = AlipayCore.paraFilter(sParaTemp);
        //生成签名结果
        String mysign = buildRequestMysign(sPara);

        //签名结果与签名方式加入请求提交参数组中
        sPara.put("sign", mysign);
        sPara.put("sign_type", cus.getAlipay().get("sign-type"));

        return sPara;
    }

    /**
     * 生成签名结果
     * @param sPara 要签名的数组
     * @return 签名结果字符串
     */
    private String buildRequestMysign(Map<String, String> sPara) {
        String prestr = AlipayCore.createLinkString(sPara); //把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串

        return RSA.sign(prestr, cus.getAlipay().get("private-key"), cus.getAlipay().get("input-charset"));
    }

}
