package com.emato.ccnet.biz.req.pay.icbc;

import com.emato.ccnet.biz.req.CebReqTransfer;
import com.emato.ccnet.config.Cus;
import com.emato.ccnet.cus.message.req.CebReqRoot;
import com.emato.ccnet.cus.message.support.CebSupport;
import com.emato.ccnet.cus.message.transfer.MessageTransfer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * 海关支付单 报文对象转XML
 * @author Scott Chen
 * @version 1.0
 * 2017-11-07 21:54
 */
@Component
public class Ceb411PayBillTransfer implements CebReqTransfer {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb411PayBillTransfer.class);

    /**
     * 是否格式化XML报文,主要用于调试程序,生产环境不能格式化(必须为false)
     */
    @Autowired
    private Cus cus;

    @Autowired
    private MessageTransfer messageTransfer;


    @Override
    public String transform(CebReqRoot msgObj, String orderNo) {
        LOGGER.info("--- 【CEB411】订单:【{}】,报文Java对象转XML字符串开始......", orderNo);
        String xmlStr="";
        try {
            //报文对象转XML字符串
            xmlStr = messageTransfer.toXml(CebSupport.CEB411, msgObj, Boolean.valueOf(cus.getMessage().get("format")));
            LOGGER.debug("--- 【CEB411】订单:【{}】,转换后的报文XML字符串:{}", orderNo, xmlStr);
        } catch (Exception e) {
            LOGGER.error("--- 【CEB411】订单:【{}】,EPort请求报文Java对象转为XML字符串时，发生错误", orderNo, e);
            return xmlStr;
        }
        LOGGER.info("--- 【CEB411】订单:【{}】,报文Java对象转XML字符串结束----------", orderNo);
        return xmlStr;
    }
}
