package com.emato.ccnet.biz.req.pay.icbc;

import com.emato.ccnet.biz.req.CebReqMessage;
import com.emato.ccnet.config.Cus;
import com.emato.ccnet.config.Merchant;
import com.emato.ccnet.cus.message.req.AbstractCebReq;
import com.emato.ccnet.cus.message.req.CebReqMessageHead;
import com.emato.ccnet.cus.message.req.CebReqRoot;
import com.emato.ccnet.cus.message.req.ceb411.Ceb411Payment;
import com.emato.ccnet.cus.message.req.ceb411.Ceb411PaymentHead;
import com.emato.ccnet.cus.message.req.ceb411.CebReq411Message;
import com.emato.ccnet.cus.message.support.CebSupport;
import com.emato.ccnet.domain.kmall.PayBill;
import com.emato.ccnet.util.LocalDateTimeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.UUID;

/**
 * 海关支付单 报文Java对象生成
 * @author Scott Chen
 * @version 1.0
 * 2017-11-07 09:49
 */
@Component
public class CebReq411PayBillMessage extends AbstractCebReq implements CebReqMessage {
    private static final Logger LOGGER = LoggerFactory.getLogger(CebReq411PayBillMessage.class);

    @Autowired
    private Cus cus;
    @Autowired
    private Merchant merchant;

    /**
     * 报文生成
     * @param messageType
     * @param bizParams
     * @param <T>
     * @return
     */
    @Override
    public <T extends CebReqRoot> T buildMessage(Map<String, Object> bizParams, String messageType, String orderNo) {
        LOGGER.info("--- 订单:【{}】, 报文Java对象生成开始......", orderNo);

        PayBill payBill = (PayBill)bizParams.get("payBill");
        //报文生成异常时,记录文件名到库
        String msgFileName = (String) bizParams.get("msgFileName");

        //---------- 报文组装 ------------------------------

        //----------报文----------
        CebReq411Message ceb411Message = super.setMessageAttribute(new CebReq411Message());

        /**
         * 报文编号
         * 系统唯一序号
         * 企业系统生成36 位单证唯一序号（要求为guid36 位，英文字母大写）
         */
        String messageId = UUID.randomUUID().toString().toUpperCase();
        String guid = UUID.randomUUID().toString().toUpperCase();

        ceb411Message.setGuid(messageId);

        //----------报文头----------
        CebReqMessageHead cebReqMessageHead = super.setMessageHead(new CebReqMessageHead());
        //报文唯一编号
        cebReqMessageHead.setMessageID(messageId);
        //报文类型
        cebReqMessageHead.setMessageType(messageType);
        /**
         * 接收部门
         * 填写本报文发送的监管单位，可同时填写多个监管部门：
         * C-海关；Q-检验检疫；
         * 例如：同时发送至海关、检验检疫可填写：CQ
         * 本节点根据监管部门要求将来可扩展
         */
        cebReqMessageHead.setReceiverDepartment(CebSupport.ceb_cq);


        //----------报文体----------
        Ceb411Payment ceb411Payment = new Ceb411Payment();

        //----------表头----------
        Ceb411PaymentHead ceb411PaymentHead = new Ceb411PaymentHead();
        ceb411PaymentHead.setGuid(guid);
        ceb411PaymentHead.setAppType(CebSupport.APP_TYPE_ADD);
        ceb411PaymentHead.setAppStatus(CebSupport.APP_status_decl);
        ceb411PaymentHead.setAppTime(LocalDateTimeUtils.formatNow("yyyyMMddHHmmss"));


        ceb411PaymentHead.setPayCode(cus.getAlipay().get("copCode"));
        ceb411PaymentHead.setPayName(cus.getAlipay().get("copName"));

        //支付编号
        ceb411PaymentHead.setPayTransactionId(payBill.getTradeNo());
        //订单编号
        ceb411PaymentHead.setOrderNo(payBill.getOrderNo());
        //电商平台代码
        ceb411PaymentHead.setEbpCode(merchant.getMerchantCustomsCode());
        //电商平台名称
        ceb411PaymentHead.setEbpName(merchant.getMerchantCustomsName());
        //支付人证件类型
        ceb411PaymentHead.setPayerIdType(payBill.getIdType());
        //支付人证件号码
        ceb411PaymentHead.setPayerIdNumber(payBill.getIdCode());
        //支付人姓名
        ceb411PaymentHead.setPayerName(payBill.getTrueName());
        //支付人电话
         ceb411PaymentHead.setTelephone(payBill.getMobile());
        //支付金额
        ceb411PaymentHead.setAmountPaid(payBill.getRealAmount().toString());
        //支付币制
        ceb411PaymentHead.setCurrency(payBill.getCurrency());

        //支付时间
        ceb411PaymentHead.setPayTime(LocalDateTimeUtils.formatTime(payBill.getPayTime(), "yyyyMMddHHmmss"));

        //备注, 与工行商定填写支付渠道
        ceb411PaymentHead.setNote(cus.getAlipay().get("payChannel"));

        //----------表头添加到报文体----------
        ceb411Payment.setCEB411PaymentHead(ceb411PaymentHead);

        //----------报文头添加到报文----------
        ceb411Message.setCebReqMessageHead(cebReqMessageHead);
        //----------报文体添加到报文----------
        ceb411Message.setCEB411Payment(ceb411Payment);

        LOGGER.info("--- 订单:【{}】, 支付单:【{}】,生成报文对象数据: {}", payBill.getOrderNo(), payBill.getTradeNo(), ceb411Message.toString());
        return (T)ceb411Message;
    }
}
