package com.emato.ccnet.biz.req.waybill.oms;

import com.emato.ccnet.biz.req.CebReqSendDate;
import com.emato.ccnet.config.Cus;
import com.emato.ccnet.contants.Contants;
import com.emato.ccnet.domain.ccnet.CusWaybill;
import com.emato.ccnet.domain.ccnet.CusWaybillGoods;
import com.emato.ccnet.domain.ccnet.CusWaybillStatus;
import com.emato.ccnet.dto.BondedStockMerchantOrder;
import com.emato.ccnet.dto.BondedStockMerchantOrderItem;
import com.emato.ccnet.msg.ResponseMessage;
import com.emato.ccnet.service.ccent.CusWaybillGoodsService;
import com.emato.ccnet.service.ccent.CusWaybillService;
import com.emato.ccnet.service.ccent.CusWaybillStatusService;
import com.emato.ccnet.util.JacksonUtil;
import com.emato.ccnet.util.OkHttpUtil;
import com.emato.ccnet.util.sign.OmsSign;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.*;

@Component
public class Ceb511SendDateRequest implements CebReqSendDate{
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb511SendDateRequest.class);
    @Autowired
    private CusWaybillService cusWaybillService;

    @Autowired
    private CusWaybillStatusService cusWaybillStatusService;

    @Autowired
    private CusWaybillGoodsService cusWaybillGoodsService;

    @Autowired
    private Ceb511SendDateResponse ceb511SendDateResponse;

    @Autowired
    private Ceb511SendDateAfter ceb511SendDateAfter;

    @Autowired
    private Cus cus;

    /**
     * 发送请求
     * @return
     */
    @Override
    public ResponseMessage sendDate() {
        //查找当前ccnet中的所有电子订单
        List<CusWaybill> cusWaybills = cusWaybillService.findAllCusWaybill();
        if(cusWaybills == null){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送给第三方的请求回调
        int orderSumbit = buildRequest(cusWaybills);
        if(orderSumbit != 1){
            String info = "发送请求响应失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }
        return ResponseMessage.success();
    }

    /**
     * 发送第三方请求
     * @param cusWaybills
     * @return
     */
    private int buildRequest(List<CusWaybill> cusWaybills) {

        List<String> sucList = new ArrayList<>();
        List<String> failedList = new ArrayList<>();

        for (CusWaybill cusWaybill:cusWaybills){
            //更新运单状态表变成发送中
            ResponseMessage update = updateCusWaybillStatus(cusWaybill.getCusWaybillSn());
            if(!ResponseMessage.isSuccess(update)){
                failedList.add(cusWaybill.getCusWaybillSn());
                String info = "更新运单状态表变成发送中失败" + update.getMsg();
                LOGGER.error("--- {}", info);
                continue;
            }

            //构造请求oms的参数数值
            Map<String, String> sPara = buildCusWaybillMap(cusWaybill);
            if(sPara == null){
                failedList.add(cusWaybill.getCusWaybillSn());
                String info = "构造请求oms的参数数值失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //发送请求并获取响应信息
            String response = requestSubmit(sPara);
            LOGGER.info("oms的响应报文：" + response);
            if(response == null){
                failedList.add(cusWaybill.getCusWaybillSn());
                String info = "发送请求并获取响应信息失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            LOGGER.info("response date:"+response);
            //解析响应信息并处理
            ResponseMessage addRespone = ceb511SendDateResponse.addResponse(response,cusWaybill.getCusWaybillSn());
            if(!ResponseMessage.isSuccess(addRespone)){
                failedList.add(cusWaybill.getCusWaybillSn());
                String info = "解析响应信息并处理失败:"+addRespone.getMsg();
                LOGGER.error("--- {}", info);
                continue;
            }

            sucList.add(cusWaybill.getCusWaybillSn());

        }
        LOGGER.info("--- 【CEB411】【成功】订单数:{}, 订单: {}", sucList.size(), Joiner.on(",").skipNulls().join(sucList.toArray()));
        LOGGER.info("--- 【CEB411】【失败】订单数:{}, 订单: {}", failedList.size(), Joiner.on(",").skipNulls().join(failedList.toArray()));

        ceb511SendDateAfter.afterSend(sucList,failedList);

        return 1;
    }

    /**
     * 发送请求并获取响应信息
     * @param sPara
     * @return
     */
    private String requestSubmit(Map<String, String> sPara) {
        //构造RequestBody
        RequestBody requestBody = OkHttpUtil.buildRequestBody(sPara);

        //构建Request
        String url = cus.getWaybill().get("waybill-url");
        Request request = OkHttpUtil.buildRequest(url,requestBody);
        LOGGER.info("oms的请求报文：" + request);
        // 同步访问,返回结果字符串
        String response = null;
        try {
            response = OkHttpUtil.post(request);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return response;
    }


    /**
     * 构造请求参数
     * @param cusWaybill
     * @return
     */
    private Map<String,String> buildCusWaybillMap(CusWaybill cusWaybill) {
        Map<String,String> sParaTemp = new TreeMap<String, String>();
        //组装body数据
        BondedStockMerchantOrder bondedStockMerchantOrder = new BondedStockMerchantOrder(cusWaybill);
        List<CusWaybillGoods> cusWaybillGoodsList = cusWaybillGoodsService.findCusWaybillGoodsByOrderSn(cusWaybill.getMerchOrderSn());
        if(cusWaybillGoodsList == null||cusWaybillGoodsList.size() <= 0){
            return null;
        }
        List<BondedStockMerchantOrderItem> bondedStockMerchantOrderItems = new ArrayList<>();
        for (CusWaybillGoods cusWaybillGoods:cusWaybillGoodsList){
            BondedStockMerchantOrderItem bondedStockMerchantOrderItem = new BondedStockMerchantOrderItem();
            bondedStockMerchantOrderItem.setSku(cusWaybillGoods.getSku());
            bondedStockMerchantOrderItem.setSellQty(cusWaybillGoods.getBuyQty());
            bondedStockMerchantOrderItem.setSellUnitPrice(cusWaybillGoods.getBuyUnitPrice());
            bondedStockMerchantOrderItems.add(bondedStockMerchantOrderItem);
        }
        bondedStockMerchantOrder.setItem(bondedStockMerchantOrderItems);
        String bondedStockMerchantOrderJson = JacksonUtil.toJson(bondedStockMerchantOrder);

        String timestamp = String.valueOf(System.currentTimeMillis()/1000);
        sParaTemp.put("merchId",cusWaybill.getMerchSn());
        sParaTemp.put("data",bondedStockMerchantOrderJson);
        sParaTemp.put("timestamp", timestamp);
        //生成要请求给oms秘钥
        String sign = OmsSign.sign(sParaTemp,cus.getWaybill().get("secret-key"));
        sParaTemp.put("sign", sign);


        LOGGER.info("oms的请求报文：merchId参数:" + cusWaybill.getMerchSn()+"sign参数:"+sign+"data参数:"+bondedStockMerchantOrderJson+"timestamp参数:"+timestamp);
        return sParaTemp;
    }

    /**
     * 利用Introspector和PropertyDescriptor 将Bean --> Map
     * @param obj
     * @return
     */
    public static Map<String, Object> transBean2Map(Object obj) {

        if(obj == null){
            return null;
        }
        Map<String, Object> map = new HashMap<String, Object>();
        try {
            BeanInfo beanInfo = Introspector.getBeanInfo(obj.getClass());
            PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
            for (PropertyDescriptor property : propertyDescriptors) {
                String key = property.getName();

                // 过滤class属性
                if (!key.equals("class")) {
                    // 得到property对应的getter方法
                    Method getter = property.getReadMethod();
                    Object value = getter.invoke(obj);

                    map.put(key, value);
                }

            }
        } catch (Exception e) {
            System.out.println("transBean2Map Error " + e);
        }

        return map;

    }

    /**
     * 更新运单状态
     * @param cusWaybillSn
     * @return
     */
    private ResponseMessage updateCusWaybillStatus(String cusWaybillSn) {
        Map<String, Object> map = ImmutableMap.of("cusWaybillSn", cusWaybillSn, "sendStatus", Contants.INIT_STATUS);
        //通过编号查找运单状态表
        CusWaybillStatus cusWaybillStatus = cusWaybillStatusService.findCusWaybillStatus(map);
        if(cusWaybillStatus == null){
            return  ResponseMessage.failed("查找运单状态表为空");
        }

        //更新支付单阿里状态
        cusWaybillStatus.setSendStatus(Contants.SENDING_STATUS);
        cusWaybillStatus.setModerSn(Contants.CREATER_NUMBER);
        cusWaybillStatus.setModTime(new Date());

        if(cusWaybillStatusService.updateCusWaybillStatus(cusWaybillStatus) <= 0){
          return ResponseMessage.failed("更新支付单阿里状态失败");
        }
        return ResponseMessage.success();
    }

}
