package com.emato.ccnet.config.db.primary;

import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableList;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.PropertyResolver;


/**
 * Mybatis Mapper接口扫描配置
 *
 * @author Scott Chen
 * @date 2017/4/23
 */
@Configuration
//由于MapperScannerConfigurer执行的比较早，所以必须有下面的注解
@AutoConfigureAfter(PrimaryDataSourceConfig.class)
public class PrimaryMapperConfig {

    @Autowired
    private PropertyResolver propertyResolver;

    @Bean
    public MapperScannerConfigurer primaryMapperScannerConfigurer() {
        MapperScannerConfigurer mapperScannerConfigurer = new MapperScannerConfigurer();
        //Mapper接口包
        mapperScannerConfigurer.setBasePackage(mapperPackage());
        //要使用的数据源
        mapperScannerConfigurer.setSqlSessionTemplateBeanName("primarySqlSessionTemplate");
        return mapperScannerConfigurer;
    }

    /**
     * 添加要扫描的Mapper接口包
     * 以逗号分隔的字符串
     *
     * @return
     */
    protected String mapperPackage() {
        ImmutableList.Builder<String> builder = new ImmutableList.Builder();
        builder.add("com.emato.ccnet.mapper.kamll");
        return Joiner.on(",").join(builder.build());
    }

}
