package com.emato.ccnet.config.jackson;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalTimeSerializer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;

/**
 * @author Scott Chen
 * @date 2017/4/19
 */
@Configuration
public class JacksonObjectMapper {

    @Bean
    public ObjectMapper objectMapper() {
        ObjectMapper objectMapper = new ObjectMapper();
        //序列化时,如果没有为类型找到访问者,则会抛出异常以将其指定为非可序列化类型; 如果禁用，它们将被序列化为空对象
        objectMapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        //反序列化时,遇到未知属性是否抛JsonMappingException异常.默认JsonMappingException
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        //设置null值不参与序列化(字段不被显示)
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);

        // register modules

        //序列化LocalDateTime时间
        objectMapper.registerModule(dateSerModule());
        //反序列化LocalDateTime时间
        objectMapper.registerModule(dateDeserModule());

        //序列化Unicode编码非ASCII字符
        //objectMapper.registerModule(unicodeSerModule());

        return objectMapper;
    }

    /**
     * 序列化LocalDateTime时间
     * @return
     */
    @Bean
    public SimpleModule dateSerModule() {
        SimpleModule dateSerModule = new SimpleModule();
        dateSerModule.addSerializer(LocalDate.class, new LocalDateSerializer(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        dateSerModule.addSerializer(LocalTime.class, new LocalTimeSerializer(DateTimeFormatter.ofPattern("HH:mm:ss")));
        dateSerModule.addSerializer(LocalDateTime.class, new LocalDateTimeSerializer(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        return dateSerModule;
    }

    /**
     * 反序列化LocalDateTime时间
     * @return
     */
    @Bean
    public SimpleModule dateDeserModule() {
        SimpleModule dateDeserModule = new SimpleModule();
        dateDeserModule.addDeserializer(LocalDate.class, new LocalDateDeserializer(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        dateDeserModule.addDeserializer(LocalTime.class, new LocalTimeDeserializer(DateTimeFormatter.ofPattern("HH:mm:ss")));
        dateDeserModule.addDeserializer(LocalDateTime.class, new LocalDateTimeDeserializer(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        return dateDeserModule;
    }

    /**
     * 序列化Unicode编码非ASCII字符
     */
    @Bean
    public SimpleModule unicodeSerModule() {
        SimpleModule unicodeSerModule = new SimpleModule();
        unicodeSerModule.addSerializer(String.class, new JacksonStringUnicodeSerializer());
        return unicodeSerModule;
    }

}
