package com.emato.ccnet.config.servlet;

import com.google.common.collect.ImmutableList;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.web.filter.CharacterEncodingFilter;
import org.springframework.web.servlet.ViewResolver;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;
import org.springframework.web.servlet.resource.DefaultServletHttpRequestHandler;
import org.springframework.web.servlet.view.InternalResourceViewResolver;

import java.util.List;

/**
 * @author Scott Chen
 * @version 1.0
 * 2017-11-21 19:06
 */
@Configuration
public class WebMvcConfig extends WebMvcConfigurerAdapter {

    /**
     * 编码过滤器
     * @return
     */
    @Bean
    public FilterRegistrationBean characterEncodingFilter() {
        CharacterEncodingFilter filter = new CharacterEncodingFilter();
        filter.setEncoding("UTF-8");
        filter.setForceEncoding(true);

        FilterRegistrationBean filterBean = new FilterRegistrationBean();
        filterBean.setFilter(filter);
        filterBean.addUrlPatterns("/*");
        return filterBean;
    }

    /**
     * 对进入DispatcherServlet的URL进行筛查
     * 如果发现是静态资源的请求，就将该请求转由Web应用服务器默认的Servlet处理
     * 如果不是静态资源的请求，才由DispatcherServlet继续处理
     * @return
     */
    @Bean
    public DefaultServletHttpRequestHandler defaultServletHttpRequestHandler() {
        return new DefaultServletHttpRequestHandler();
    }


    /**
     * 采用jackson进行消息转换时,处理中文乱码
     * @return
     */
    @Bean
    public List<HttpMessageConverter<?>> converters() {
        StringHttpMessageConverter messageConverter = new StringHttpMessageConverter();
        //处理中文乱码
        ImmutableList<MediaType> list = ImmutableList.of(MediaType.TEXT_HTML, MediaType.APPLICATION_JSON_UTF8);
        messageConverter.setSupportedMediaTypes(list);

        return ImmutableList.of(messageConverter);
    }

    /**
     * 配置处理jackson消息转器
     * @param converters
     */
    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
        super.configureMessageConverters(converters);
    }


    /**
     * 注册视图处理器
     * @return
     */
    @Bean
    public ViewResolver viewResolver() {
        InternalResourceViewResolver viewResolver = new InternalResourceViewResolver();
        viewResolver.setPrefix("");
        viewResolver.setSuffix(".html");
        return viewResolver;
    }


    /**
     * 仅限于嵌入式Tomcat，配置自定义404等常见错误处理页面,部署为外置War包时,不适用
     * @return
     */
    /*@Bean
    public EmbeddedServletContainerCustomizer containerCustomizer() {
        return (container -> {
            ErrorPage error401Page = new ErrorPage(HttpStatus.UNAUTHORIZED, "/401.html");
            ErrorPage error404Page = new ErrorPage(HttpStatus.NOT_FOUND, "/404.html");
            ErrorPage error500Page = new ErrorPage(HttpStatus.INTERNAL_SERVER_ERROR, "/500.html");

            container.addErrorPages(error401Page, error404Page, error500Page);
        });
    }*/

}
