package com.emato.ccnet.cus.message.req;

import com.emato.ccnet.config.Cus;
import com.emato.ccnet.cus.message.support.CebSupport;
import com.emato.ccnet.util.LocalDateTimeUtils;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * EPort请求报文生成抽象类
 * 提供请求报文中公共属性和字段设置
 *
 * 可以由运维配置项,在properties文件,参见{@code eport.properties}
 * 涉及程序相关配置项,参见{@link com.emato.ccnet.cus.message.support.CebSupport}
 *
 * @author Scott Chen
 * @date 2016/11/24
 */
public abstract class AbstractCebReq {

    @Autowired
    private Cus cus;

    /**
     * 设置EPort请求报文属性
     *
     * @param <T>
     * @return 返回报文根节点
     */
    public <T extends CebReqRoot> T setMessageAttribute(T root) {
        root.setXmlns(CebSupport.xmlns);
        root.setXmlnsXsi(CebSupport.xmlnsXsi);
        root.setVersion(CebSupport.version);
        return root;
    }

    /**
     * 设置EPort请求报文头
     * @return
     */
    public CebReqMessageHead setMessageHead(CebReqMessageHead cebReqMessageHead) {
        //报文头
        //messageHead.setMessageType("CEB511Message");
        //messageHead.setMessageID("4575B0EE-B081-4922-9112-E54972DA479X");
        //企业组织机构代码或统一社会信用代码
        cebReqMessageHead.setOrgCode(cus.getAlipay().get("orgCode"));
        //传输企业代码
        cebReqMessageHead.setCopCode(cus.getAlipay().get("copCode"));
        //传输企业名称
        cebReqMessageHead.setCopName(cus.getAlipay().get("copName"));
        //发送方
        cebReqMessageHead.setSenderID(cus.getAlipay().get("senderID"));
        //接收方
        cebReqMessageHead.setReceiverID(cus.getAlipay().get("receiverID"));
        //接收部门
        //messageHead.setReceiverDepartment("CQ");
        //发送时间,格式：yyyyMMddHHmmssSSS
        cebReqMessageHead.setSendTime(LocalDateTimeUtils.formatNow("yyyyMMddHHmmssSSS"));
        //版本号
        cebReqMessageHead.setVersion(CebSupport.version);
        return cebReqMessageHead;
    }

}
