package com.emato.ccnet.redis.jedis.cluster.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import redis.clients.jedis.HostAndPort;
import redis.clients.jedis.JedisCluster;

import java.util.Set;

/**
 * @author Scott Chen
 * @since 1.0
 * 2018-01-06 16:14
 */
public class JedisClusterBuilder {

    private Set<HostAndPort> jedisClusterNode;
    private int connectionTimeout;
    private int soTimeout;
    private int maxAttempts;
    private String password;
    private GenericObjectPoolConfig poolConfig;

    public JedisClusterBuilder(Set<HostAndPort> jedisClusterNode) {
        this.jedisClusterNode = jedisClusterNode;
    }

    public static JedisClusterBuilder create(Set<HostAndPort> jedisClusterNode) {
        return new JedisClusterBuilder(jedisClusterNode);
    }

    public JedisCluster build() {
        return new JedisCluster(
                this.jedisClusterNode,
                this.connectionTimeout,
                this.soTimeout,
                this.maxAttempts,
                this.password,
                this.poolConfig
        );
    }

    public JedisClusterBuilder setJedisClusterNode(Set<HostAndPort> jedisClusterNode) {
        this.jedisClusterNode = jedisClusterNode;
        return this;
    }

    public JedisClusterBuilder setConnectionTimeout(int connectionTimeout) {
        this.connectionTimeout = connectionTimeout;
        return this;
    }

    public JedisClusterBuilder setSoTimeout(int soTimeout) {
        this.soTimeout = soTimeout;
        return this;
    }

    public JedisClusterBuilder setMaxAttempts(int maxAttempts) {
        this.maxAttempts = maxAttempts;
        return this;
    }

    public JedisClusterBuilder setPassword(String password) {
        this.password = password;
        return this;
    }

    public JedisClusterBuilder setPoolConfig(GenericObjectPoolConfig poolConfig) {
        this.poolConfig = poolConfig;
        return this;
    }
}
