package com.emato.ccnet.redis.jedis.cluster.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import redis.clients.jedis.HostAndPort;

import java.io.Serializable;
import java.util.*;

/**
 * @author Scott Chen
 * @since 1.0
 * 2018-01-06 16:12
 */
public class JedisClusterProperties implements Serializable {
    private static final Logger logger = LoggerFactory.getLogger(JedisClusterProperties.class);
    private static final long serialVersionUID = -1621540040447290980L;

    // 配置文件中主机节点
    private List<RedisClusterHost> redisClusterHost;
    private int connectionTimeout;
    private int soTimeout;
    private int maxAttempts;
    private String password;
    private GenericObjectPoolConfig poolConfig;

    // Jedis cluster 主机节点
    private Set<HostAndPort> jedisClusterNode = new LinkedHashSet<>();

    public JedisClusterBuilder initializeJedisClusterBuilder() {
        buildJedisClusterNode();
        return JedisClusterBuilder.create(this.jedisClusterNode)
                .setConnectionTimeout(this.connectionTimeout)
                .setSoTimeout(this.soTimeout)
                .setMaxAttempts(this.maxAttempts)
                .setPassword(this.password)
                .setPoolConfig(this.poolConfig);
    }

    /**
     * 配置文件中主机节点转为Jedis cluster 主机节点
     * @return
     */
    public Set<HostAndPort> buildJedisClusterNode() {
        if (this.redisClusterHost == null || redisClusterHost.isEmpty()) {
            throw new IllegalArgumentException("Redis Cluster Hosts are null");
        }
        try {
            redisClusterHost
                .stream()
                .forEach( x -> {
                    jedisClusterNode.add(new HostAndPort(x.getHost(), Integer.valueOf(x.getPort())));
                    logger.info("Redis Cluster Host, IP:{}, PORT:{}", x.getHost(), x.getPort());
                });
        } catch (Exception e) {
            throw new RuntimeException("Redis Cluster Hosts convert error", e);
        }

        if (jedisClusterNode == null || jedisClusterNode.isEmpty()) {
            return Collections.emptySet();
        }
        return jedisClusterNode;
    }

    public List<RedisClusterHost> getRedisClusterHost() {
        return redisClusterHost;
    }

    public void setRedisClusterHost(List<RedisClusterHost> redisClusterHost) {
        this.redisClusterHost = redisClusterHost;
    }

    public int getConnectionTimeout() {
        return connectionTimeout;
    }

    public void setConnectionTimeout(int connectionTimeout) {
        this.connectionTimeout = connectionTimeout;
    }

    public int getSoTimeout() {
        return soTimeout;
    }

    public void setSoTimeout(int soTimeout) {
        this.soTimeout = soTimeout;
    }

    public int getMaxAttempts() {
        return maxAttempts;
    }

    public void setMaxAttempts(int maxAttempts) {
        this.maxAttempts = maxAttempts;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public GenericObjectPoolConfig getPoolConfig() {
        return poolConfig;
    }

    public void setPoolConfig(GenericObjectPoolConfig poolConfig) {
        this.poolConfig = poolConfig;
    }


    /**
     * RedisClusterHosts
     */
    public static class RedisClusterHost implements Serializable{
        private static final long serialVersionUID = 1527946299604181444L;
        private String host;
        private String port;

        public String getHost() {
            return host;
        }

        public void setHost(String host) {
            this.host = host;
        }

        public String getPort() {
            return port;
        }

        public void setPort(String port) {
            this.port = port;
        }
    }

}
