package com.emato.ccnet.redis.jedis.cluster.exception;

import com.emato.ccnet.msg.ResponseMessage;
import com.emato.ccnet.redis.jedis.cluster.msg.JedisStatusMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import redis.clients.jedis.exceptions.JedisNoReachableClusterNodeException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Jedis 异常处理
 * @author Scott Chen
 * @since 1.0
 * 2018-01-12
 */
@ControllerAdvice
public class JedisExceptionHandler {
    private static final Logger logger = LoggerFactory.getLogger(JedisExceptionHandler.class);

    @ExceptionHandler(JedisNoReachableClusterNodeException.class)
    @ResponseBody
    public ResponseMessage handleJedisException(HttpServletRequest request, HttpServletResponse response,
                                                JedisNoReachableClusterNodeException ex) {

        Map<String, Object> model = new ConcurrentHashMap<String, Object>();
        model.put("ex", ex);
        // 可以细化异常信息，给出相应的提示
        logger.info("==========Jedis Exception异常");
        logger.info("==========异常类型："+ex.getClass().getSimpleName());
        logger.info("==========异常描述："+ex.getMessage());
        logger.info("==========异常原因："+ex.getCause());

        return ResponseMessage.error(JedisStatusMessage.I_100000.CODE, JedisStatusMessage.I_100000.INFO);
        //return "500";
    }
}
